@@ -1,8 +1,5 @@
 package Apache::AuthCookie::Util;
-{
-  $Apache::AuthCookie::Util::VERSION = '3.19';
-}
-
+$Apache::AuthCookie::Util::VERSION = '3.21';
 # ABSTRACT: Internal Utility Functions for AuthCookie
 
 use strict;
@@ -81,7 +78,8 @@ sub escape_destination {
 sub understands_forbidden_response {
     my $ua = shift;
 
-    return 0 if $ua =~ qr{\AMozilla/5\.0 \(SymbianOS/};
+    return 0 if $ua =~ qr{\AMozilla/5\.0 \(SymbianOS/}  # Symbian phones
+             or $ua =~ qr{\bIEMobile/10};            # Nokia Lumia 920, possibly others?
 
     return 1;
 }
@@ -98,7 +96,7 @@ Apache::AuthCookie::Util - Internal Utility Functions for AuthCookie
 
 =head1 VERSION
 
-version 3.19
+version 3.21
 
 =head1 SOURCE
 
